@@ -23,6 +23,7 @@ design rationale:
 */
 
 static int cflag;
+static int kflag;
 static int Mflag;
 static int vflag;
 static char *Xflag;
@@ -63,8 +64,7 @@ deliver(FILE *infile)
 			ssize_t rd = getdelim(&line, &linelen, '\n', infile);
 			if (rd == -1) {
 				if (errno == 0)
-					// invalid mbox file
-					errno = EINVAL;
+					errno = EINVAL;  // invalid mbox file
 				return -1;
 			}
 
@@ -88,6 +88,8 @@ tryagain:
 		if (outfd < 0) {
 			if (errno == EEXIST)
 				goto tryagain;
+			fprintf(stderr, "mrefile: %s: %s\n",
+			    tmp, strerror(errno));
 			return -1;
 		}
 
@@ -107,22 +109,24 @@ tryagain:
 			}
 			char *line_start = line;
 
-			if (line[0] == '\n' && !line[1])
+			if (line[0] == '\n' && (!line[1] ||
+			                        (line[1] == '\r' && !line[2])))
 				in_header = 0;
 
 			if (Mflag && strncmp("From ", line, 5) == 0)
 				break;
 
 			if (Mflag && in_header &&
-			    (strncasecmp("status:", line, 6) == 0 ||
-			     strncasecmp("x-status:", line, 8) == 0)) {
+			    (strncasecmp("status:", line, 7) == 0 ||
+			     strncasecmp("x-status:", line, 9) == 0)) {
 				char *v = strchr(line, ':');
-				if (strchr(v, 'F')) statusflags[0] = 'F';
-				if (strchr(v, 'A')) statusflags[1] = 'R';
-				if (strchr(v, 'R')) statusflags[2] = 'S';
-				if (strchr(v, 'D')) statusflags[3] = 'T';
-				if (strchr(v, 'O')) is_old = 1;
-
+				if (v) {
+					if (strchr(v, 'F')) statusflags[0] = 'F';
+					if (strchr(v, 'A')) statusflags[1] = 'R';
+					if (strchr(v, 'R')) statusflags[2] = 'S';
+					if (strchr(v, 'D')) statusflags[3] = 'T';
+					if (strchr(v, 'O')) is_old = 1;
+				}
 				continue;  // drop header
 			}
 
@@ -183,9 +187,69 @@ tryagain:
 	return 0;
 }
 
+void
+refile(char *file)
+{
+	while (*file == ' ' || *file == '\t')
+		file++;
+
+	FILE *f = fopen(file, "r");
+	if (!f) {
+		fprintf(stderr, "mrefile: %s: %s\n", file, strerror(errno));
+		return;
+	}
+
+	// keep flags
+	char *flags = strstr(file, ":2,");
+	if (flags)
+		Xflag = flags + 3;
+	else
+		Xflag = "";
+
+	if (deliver(f) < 0) {
+		perror("mrefile");
+		return;
+	}
+
+	fclose(f);
+	if (!kflag)
+		unlink(file);
+}
+
 int
 main(int argc, char *argv[])
 {
+	if (strchr(argv[0], 'f')) {
+		// mrefile(1)
+
+		cflag = 1;  // use cur/
+
+		int c;
+		while ((c = getopt(argc, argv, "kv")) != -1)
+			switch (c) {
+			case 'k': kflag = 1; break;
+			case 'v': vflag = 1; break;
+			default:
+usage:
+				fprintf(stderr,
+				    "Usage: mrefile [-kv] [msgs...] maildir\n");
+			exit(1);
+		}
+
+		if (argc == optind)
+			goto usage;
+
+		targetdir = argv[argc - 1];
+		gethost();
+
+		if (argc == optind + 1 && isatty(0))
+			goto usage;
+		else
+			blaze822_loop(argc - 1 - optind, argv + optind, refile);
+
+		return 0;
+	}
+
 	int c;
 	while ((c = getopt(argc, argv, "cMvX:")) != -1)
 		switch (c) {
